@@ -2,9 +2,12 @@
 //!
 //! `crosstown_bus` is an easy-to-configure bus in Rust with RabbitMQ for event-driven systems.
 
-use std::{error::Error, thread};
+use std::{error::Error, thread, sync::Arc, rc::Rc, borrow::{BorrowMut, Borrow}, collections::HashMap};
+use futures::AsyncReadExt;
+use serde::__private::from_utf8_lossy;
+use serde_json::json;
 
-use amiquip::{Connection, QueueDeclareOptions, ConsumerMessage, ConsumerOptions, FieldTable, ExchangeDeclareOptions, Publish};
+use amiquip::{Connection, QueueDeclareOptions, ConsumerMessage, ConsumerOptions, FieldTable, ExchangeDeclareOptions, Publish, Delivery};
 
 type HandleResult = Result<(), Box<dyn Error>>;
 type SubscribeResult = Result<(), Box<dyn Error>>;
@@ -15,86 +18,49 @@ pub struct Bus {
 }
 
 impl Bus {
-    /// Creates a Bus object using Rabbit as Event Broker.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use crosstown_bus::Bus;
-    /// let bus = Bus::new_rabbit_bus("amqp://guest:guest@localhost:5672".to_string()).unwrap();
-    /// ```
+    
     pub fn new_rabbit_bus(url: String) -> Result<Bus, Box<dyn Error>> {
         Ok(Bus { url })
     }
 
-    /// Publishes an event.
-    ///
-    /// # Examples
+    /// Subscribes to an event. 
+    /// The action name represents the action that will be taken after the message is received. It will also be used as queue name.
+    /// Multiple queues can be connected to the same exchange (abstracted as Event in this case) and all of them will receive a copy of the message.
+    ///  /// Publishes an event.
     ///
+    /// # Example
     /// ```
-    /// use crosstown_bus::Bus;
+    /// use crosstown_bus::{Bus};
+    /// use serde::Serialize;
+    /// use serde::Deserialize;
     /// 
+    /// #[derive(Serialize, Deserialize)]
+    /// pub struct UserCreated {
+    ///     name: String,
+    ///     id: String
+    /// }
+
     /// let bus = Bus::new_rabbit_bus("amqp://guest:guest@localhost:5672".to_string()).unwrap();
 
-    /// let _ = bus.subscribe_event(String::from("user_created"), String::from("send_email"), |message| {
-    ///     println!("User CREATED email sent now: {}", message);
-    ///     (false, Ok(()))
-    /// });
-
-    /// let _ = bus.subscribe_event(String::from("user_updated"), String::from("send_email"), |message| {
-    ///     println!("User updated email sent now: {}", message);
-    ///     (false, Ok(()))
-    /// });
-
-    /// let _ = bus.subscribe_event(String::from("user_updated"), String::from("update_database"), |message| {
-    ///     println!("Database Updated now: {}", message);
-    ///     (false, Ok(()))
-    /// });
-
-    /// let _ = bus.publish_event(String::from("user_created"), String::from("Paolo"));
-    /// let _ = bus.publish_event(String::from("user_updated"), String::from("Paolo Victor"));
-    /// let _ = bus.publish_event(String::from("user_created"), String::from("Thayna"));
-    /// ```
-    pub fn publish_event(&self, event_name: String, message: String) -> PublishResult {
-        let url = self.url.to_owned();
-        if let Ok(channel) = Connection::insecure_open(&url)?.open_channel(None) {
-            let publish_result = channel.basic_publish::<String>(event_name.to_owned(), Publish {
-                body: message.as_bytes(),
-                routing_key: event_name.to_owned(),
-                mandatory: false,
-                immediate: false,
-                properties: Default::default(),
-            });
-            if publish_result.is_err() {
-                return Err(Box::new(publish_result.unwrap_err()));
-            }
-        }
-        Ok(())
-    }
-
-    /// Subscribes to an event.
-    /// Receives:
-    /// - event_name: name of the event to be expected
-    /// - action_name: action that will be executed when receiving the message for that event
-    /// - handler: closure (action) that will be executed when a message is received
-    /// # Examples
-    ///
-    /// ```
-    /// use crosstown_bus::Bus;
-    /// let bus = Bus::new_rabbit_bus("amqp://guest:guest@localhost:5672".to_string()).unwrap();
-
-    /// let _ = bus.subscribe_event(String::from("user_updated"), String::from("update_database"), |message| {
-    ///     println!("Database Updated now: {}", message);
+    /// let _ = bus.subscribe_event::<UserCreated>(String::from("send_email"), |message| {
+    ///     println!("User CREATED! e-mail sent now: {}", message);
     ///     (false, Ok(()))
     /// });
     /// ```
-    pub fn subscribe_event(&self, event_name: String, action_name: String, handler: fn(message: String) -> (bool, HandleResult)) -> SubscribeResult {
-        let url = self.url.to_owned();
+    pub fn subscribe_event<'a, T: 'a>(
+        &self, 
+        action_name: String,
+        handler: fn(String) -> (bool, HandleResult)
+    ) -> SubscribeResult where T : serde::Deserialize<'a> {
+
+        let event_name = get_event_name::<T>();
 
         let mut queue_name = event_name.to_owned();
         queue_name.push_str(&String::from("."));
         queue_name.push_str(&action_name);
 
+        let url = self.url.to_owned();
+
         thread::spawn(move || {
             match Connection::insecure_open(&url) {
                 Ok(mut cnn) => {
@@ -121,9 +87,10 @@ impl Bus {
                                     for message in consumer.receiver().iter() {
                                         match message {
                                             ConsumerMessage::Delivery(delivery) => {
-                                                let body = String::from_utf8_lossy(&delivery.body);
+                                                
+                                                let final_message = String::from_utf8_lossy(&delivery.body).to_string();
                 
-                                                let handle_result = handler(body.to_string());
+                                                let handle_result = handler(final_message);
 
                                                 let retry_on_error = handle_result.0;
                                                 let result = handle_result.1;
@@ -160,19 +127,60 @@ impl Bus {
 
         Ok(())
     }
+
+
+    /// Publishes an event.
+    ///
+    /// # Example
+    /// ```
+    /// use crosstown_bus::{Bus};
+    /// use serde::Serialize;
+    /// use serde::Deserialize;
+    /// 
+    /// #[derive(Serialize, Deserialize)]
+    /// pub struct UserCreated {
+    ///     name: String,
+    ///     id: String
+    /// }
+
+    /// let bus = Bus::new_rabbit_bus("amqp://guest:guest@localhost:5672".to_string()).unwrap();
+
+    /// let res = bus.publish_event::<UserCreated>(UserCreated {
+    ///     name: "Paolo".to_owned(),
+    ///     id: "F458asYfj".to_owned()
+    /// });
+    /// ```
+    pub fn publish_event<T>(&self, message: T) -> PublishResult 
+        where T: serde::Serialize {
+        let url = self.url.to_owned();
+        let event_name = get_event_name::<T>();
+
+        let message_json = json!(message).to_string();
+
+        if let Ok(channel) = Connection::insecure_open(&url)?.open_channel(None) {
+            let publish_result = channel.basic_publish::<String>(event_name.to_owned(), Publish {
+                body: message_json.as_bytes(),
+                routing_key: event_name.to_owned(),
+                mandatory: false,
+                immediate: false,
+                properties: Default::default(),
+            });
+            if publish_result.is_err() {
+                return Err(Box::new(publish_result.unwrap_err()));
+            }
+        }
+        Ok(())
+    }
 }
 
-// #[cfg(test)]
-// mod tests {
-//     use crate::Bus;
+fn get_deserialized_message<'a, T: 'a>(message: &'a String) -> T 
+    where T : serde::Deserialize<'a> + 'a {
+        serde_json::from_str(&message).unwrap()
+}
 
-//     #[test]
-//     fn new_bus_works() {
-//         let bus = Bus::new_rabbit_bus("url".to_string()).unwrap();
-//         if bus.url.len() > 0 {
-//             assert!(true);
-//         } else {
-//             assert!(false);
-//         }
-//     }
-// }
+fn get_event_name<T>() -> String {
+    let full_event_name = std::any::type_name::<T>().to_string();
+    let event_array = full_event_name.split("::").collect::<Vec<&str>>();
+    let event_name = event_array.last().unwrap().to_string();
+    event_name
+}
\ No newline at end of file
